import com.apple.yellow.webobjects.*;
import com.apple.yellow.foundation.*;

/*
 Copyright (c) 1999, Apple Computer, Inc.
 All rights reserved.

 You may freely copy, distribute and reuse the code in this example.
 Apple disclaims any warranty of any kind, expressed or implied,
 as to its fitness for any particular use.
 */

// Renamed LongResponsePage to avoid conflict with existing WOExtensions class */
/* Abstract methods must be implemented in subclasses */

public abstract class LongResponsePage extends WOComponent implements Runnable {

    static String WOMetaRefreshSenderId = "WOMetaRefresh";
    private Object _status;
    private Object _result;
    private Exception _exception;
    private int _refreshInterval;
    private boolean _performingAction;
    private boolean _keepRefreshing = true;
    private boolean _cancelled;
    private boolean _done;
    private boolean _debug = false;

    public LongResponsePage()  {
        super();
        _finishInitialization();
    }

    private void _finishInitialization() {
        _status = null;
        _result = null;
        _done = false;
        _exception = null;
        _cancelled = false;
        _refreshInterval = 0; // set at a do-nothing value
        _performingAction = false;
    }


    //********************************************
    // abstract methods
    //********************************************

    public abstract void performAction();
    public abstract WOComponent refreshPage();
    public abstract WOComponent pageForResult();
    public abstract WOComponent cancelPage();

    //********************************************
    // the thread behavior method
    //********************************************

    public void run() {
        int pool = NSAutoreleasePool.push();  // note this is required for running Java threads
        WOApplication app = WOApplication.application();
        _done = false;
        String name = this.getClass().getName();
        app.debugString("<"+name+">: creating computation thread");
        // performAction() is defined abstract here and specifies
        // the user-defined behavior of the subclass implementation.
        try {
            this.performAction();
            // We need to catch the possible exceptions too.
        } catch (Exception localException) {
            this._setException(localException);
            if (app.isDebuggingEnabled()) {
                app.logString("<"+this.getClass().getName()+"> long response thread raised : "+localException.toString());
                localException.printStackTrace();
            }
        }
        app.debugString("<"+name+">: exiting computation thread");
        _done = true;
        NSAutoreleasePool.pop(pool);
    }

    public void appendToResponse(WOResponse aResponse, WOContext aContext)  {
        if (!_performingAction) {
            // executed on the first response to a user request or action
            _performingAction = true; // we're working now
            try {
                Thread t = new Thread(this);
                t.start(); // just calls the run() method so the page is in its own thread now
            } catch (Exception localException) {
                throw new RuntimeException ("<WOLongResponsePage> Exception occured while creating long response thread: "+localException.toString());
            }
        }
        // If the refreshInterval was set and we did not get a result yet, let's add the refresh header.
        // refreshing keeps going until _keepRefreshing knows differently in refresh() 
        if ((_refreshInterval!=0) && _keepRefreshing) {

            String modifiedDynamicUrl = aContext.urlWithRequestHandlerKey(WOApplication.application().componentRequestHandlerKey(), null, null);
            if (_debug) System.out.println("In appendToResponse modifiedDynamicUrl: "+modifiedDynamicUrl);

            String header = "" +_refreshInterval+ ";url=" +modifiedDynamicUrl+ "/" + aContext.session().sessionID()+ "/" +aContext.contextID()+ "." +WOMetaRefreshSenderId;
            if (_debug) System.out.println("In appendToResponse header: "+header);
            aResponse.setHeader(header,"Refresh");
        }
        super.appendToResponse(aResponse,aContext);
    }

    public WOElement invokeAction(WORequest aRequest, WOContext aContext)  {
        if (_debug) System.out.println("In invokeAction aContext.senderID(): "+aContext.senderID());
        if (aContext.senderID().equals(WOMetaRefreshSenderId)) {
            // we recognized the elementID that was set for the meta refresh
            // for a smaller granularity of control.
            // we know which action to call: return a refreshed page.
            return this.refresh();
        }
        return super.invokeAction(aRequest, aContext);
    }

    // called to update page
    // simple branch should be final
    public final WOComponent refresh() {
        if (_debug) System.out.println("In LongResponsePage refresh()");
        Exception e = this._exception();
        if (e != null) return this.pageForException(e);
        else if (_done) {
            _keepRefreshing = false;
            return this.pageForResult();
        }
        else if (_cancelled) return this.cancelPage();
        return this.refreshPage();
    }

    public WOComponent pageForException(Exception exception) {
        throw new RuntimeException("<WOLongResponsePage> Exception occured in long response thread: "+exception.toString());
    }

    //********************************************
    // accessor methods
    //********************************************

    private Exception _exception() {
        return _exception;
    }

    private void _setException(Exception anObject) {
        if (anObject != _exception) {
            synchronized(this) {
                _exception = anObject;
            }
        }
    }

    public WOComponent cancel()  {
        this.setCancelled(true);
        return this.cancelPage();
    }

    public boolean isCancelled() {
        return _cancelled;
    }

    public void setCancelled(boolean aBool) {
        if (aBool != _cancelled) {
            synchronized(this) {
                _cancelled = aBool;
            }
        }
    }

    public Object status() {
        return _status;
    }

    public void setStatus(Object anObject) {
        if (anObject != _status) {
            synchronized(this) {
                _status = anObject;
            }
        }
    }

    public void setRefreshInterval(double interval) {
        if (interval > 0) {
            _refreshInterval = (int)interval;
        } else {
            _refreshInterval = 0;
        }
    }

    public double refreshInterval() {
        return (double)_refreshInterval;
    }
}